# $Id: Mailman.pm,v 1.2 2004/06/20 19:23:07 rich Exp $

package Email::List::Subscription::Mailman;

use strict;
use Email::List::Subscription;
use Mail::Address;

sub match
{
    my $message = shift(@_);
    my $lines_ref = shift(@_);
    my @subscriptions = ();

    # GNU Mailman
    my $is_mailman = defined($message->header('X-MailMan-Version'));
    if ($is_mailman)
    {
	push(@subscriptions, _match($message, $lines_ref));
    }

    return @subscriptions;
}

sub _match
{
    my $message = shift(@_);
    my $lines_ref = shift(@_);
    my @subscriptions = ();
    my $subject;

    # Check this is a Mailman reminder mail.
    $subject = $message->header('Subject');
    if ($subject =~ m/ mailing list memberships reminder$/)
    {
	# TODO: Error reporting: no listname; no to.
	my $to = $message->header('To');
	my $date = $message->header('Date');

	# Parse the to address. Ignore multiple to addresses.
	my ($to_addr) = Mail::Address->parse($to);

	if (defined($to_addr) && defined($date))
	{
	    # TODO: Preserve case of mailbox.
	    $to = lc($to_addr->address());

	    my $has_passwords = 0;
	    my $i;

	    # TODO: Store helpful waffle?
	    for ($i = 0; $i <= $#{$lines_ref}; ++$i)
	    {
		if ($lines_ref->[$i] =~ m/Passwords for $to:/)
		{
		    $has_passwords = 1;
		    ++$i;
		    last;
		}
		elsif ($lines_ref->[$i] =~ m/Subscriptions for $to:/)
		{
		    ++$i;
		    last;
		}
	    }

	    for (; $i <= $#{$lines_ref}; ++$i)
	    {
		if ($lines_ref->[$i] !~ m/^\s*$/)
		{
		    last;
		}
	    }
	
	    # Skip headers for subscription info.
	    for(; $i <= $#{$lines_ref}; ++$i)
	    {
		if ($has_passwords)
		{
		    if (   ($lines_ref->[$i] !~ m!List\s+Password // URL!)
			&& ($lines_ref->[$i] !~ m!-+\s+-+!))
		    {
			last;
		    }
		}
		else
		{
		    if (   ($lines_ref->[$i] !~ m!List\s+// URL!)
			&& ($lines_ref->[$i] !~ m!-+\s+-+!))
		    {
			last;
		    }
		}
	    }

	    my $listname;
	    my $password;
	    my $url;

	    for (; $i <= $#{$lines_ref}; ++$i)
	    {
		next if ($lines_ref->[$i] =~ m/^\s*$/);

		if ($has_passwords)
		{
		    ($listname, $password) = split(/\s+/, $lines_ref->[$i]);

		    # Password may have wrapped to the next line.
		    if (!defined($password))
		    {
			++$i;
			$password = $lines_ref->[$i];
			$password =~ s/^\s*(\S+)\s*$/$1/;
		    }

		    ++$i;
		    $url = $lines_ref->[$i];
		}
		else
		{
		    $listname = $lines_ref->[$i];
		    ++$i;
		    $url = $lines_ref->[$i];
		}

		if (defined($listname))
		{
		    # TODO: Preserve case of mailbox.
		    #
		    # We lowercase it here for consistency with other
		    # Email::List::Subscription::*.
		    $listname = lc($listname);

		    my $subscription
			= new Email::List::Subscription($listname,
							$to,
							$Email::List::Subscription::SUBSCRIPTION,
							$date);

		    # Add other optional details.
		    if (defined($password))
		    {
			$subscription->add_password($password);
		    }

		    if (defined($url))
		    {
			$subscription->add_url($url);
		    }

		    push(@subscriptions, $subscription);
		}
	    }
	}
    }

    return @subscriptions;
}

1;
