# $Id: GUI.pm,v 1.3 2004/06/20 15:49:29 rich Exp $

package Subscripto::GUI;

use Carp;
use Gtk2 '-init';
use Gtk2::SimpleList;
use Gtk2::SimpleMenu;
use Subscripto::Version;

my $SORT_ASCENDING = 'ascending';
my $SORT_DESCENDING = 'descending';

sub new
{
    my $class = shift(@_);
    my $name = shift(@_);
    my $title = shift(@_);
    my $config = shift(@_);
    my $subscriptions = shift(@_);
    my $self = bless({}, $class);

    defined($title) || carp 'No title specified for the main window';

    # Set up.
    $self->_clear();

    # References to a Subscripto::Config and a Subscripto::Subscriptions.
    $self->{name} = $name;
    $self->{config} = $config;
    $self->{subscriptions} = $subscriptions;

    # Create the main window.
    $self->{window} = Gtk2::Window->new();
    defined($self->{window}) || carp 'Unable to create the main window';

    $self->{window}->signal_connect(destroy => sub { Gtk2->main_quit; });
    $self->{window}->set_title($title);
    $self->{window}->set_resizable(1);

    # Resize to half of the screen horizontally and three-quarters
    # vertically. Position it in the centre.
    $self->{window}->set_position('center');
    my ($rootx, $rooty) = $self->{window}->get_position();
    my ($width, $height) = $self->{window}->get_size();
    my $screenwidth = (2 * $rootx) + $width;
    my $screenheight = (2 * $rooty) + $height;
    $self->{window}->resize($screenwidth / 2, $screenheight * 3/4);
    $self->{window}->set_position('center');

    my $vbox = Gtk2::VBox->new();
    defined($vbox) || carp 'Unable to create the vbox';

    $self->{window}->add($vbox);

    # Add a menu.
    $self->{menu} = $self->_build_menu();
    defined($self->{menu}) || carp 'Unable to create the menu';

    $self->{window}->add_accel_group($self->{menu}->{accel_group});
    $vbox->pack_start($self->{menu}->{widget}, 0, 0, 0);

    # Add a toolbar.
    my $toolbar = Gtk2::Toolbar->new();
    defined($toolbar) || carp 'Unable to create the toolbar';

    $vbox->pack_start($toolbar, 0, 0, 0);

    # Create a scrolled window and add a simple list to it.
    my $scrolledwindow = Gtk2::ScrolledWindow->new();
    defined($scrolledwindow) || carp 'Unable to create the scrolled window';

    $vbox->pack_start($scrolledwindow, 1, 1, 0);

    $self->{slist} = Gtk2::SimpleList->new('List' => 'text',
					   'Address' => 'text',
					   'Subscribed' => 'bool');
    defined($self->{slist}) || carp 'Unable to create the list';

    $scrolledwindow->add($self->{slist});

    # Add a status bar.
    $self->{statusbar} = Gtk2::Statusbar->new();
    defined($self->{statusbar}) || carp 'Unable to create the status bar';

    $vbox->pack_start($self->{statusbar}, 0, 0, 0);

    # Set up the toolbar.
    $toolbar->insert_stock('gtk-goto-top',
			   'Go to first',
			   undef,
			   \&_goto_clicked,
			   [ $self, 'top' ],
			   0);

    $toolbar->insert_stock('gtk-go-up',
			   'Go to previous',
			   undef,
			   \&_goto_clicked,
			   [ $self, 'up' ],
			   1);

    $toolbar->insert_stock('gtk-go-down',
			   'Go to next',
			   undef,
			   \&_goto_clicked,
			   [ $self, 'down' ],
			   2);

    $toolbar->insert_stock('gtk-goto-bottom',
			   'Go to last',
			   undef,
			   \&_goto_clicked,
			   [ $self, 'bottom' ],
			   3);

    $toolbar->insert_stock('gtk-properties',
			   'Properties',
			   undef,
			   \&_show_properties,
			   $self,
			   4);

    # Set up the simple list.
    $self->{slist}->set_rules_hint(1);
    # TODO: Allow searches.
    #$self->{slist}->set_enable_search(1);
    #$self->{slist}->set_searchable_column(1);
    #$self->{slist}->set_search_equal_func( { $self->{slist}[] =~ /

    my @columns = $self->{slist}->get_columns();

    $columns[0]->set_sort_indicator(1);
    $columns[0]->set_sort_order($SORT_ASCENDING);

    $columns[0]->set_clickable(1);
    $columns[0]->signal_connect(clicked => \&_column_clicked, $self);

    $columns[1]->set_clickable(1);
    $columns[1]->signal_connect(clicked => \&_column_clicked, $self);

    # TODO: Pop-up properties menu.
    #my $prop_menu = build_prop_menu();

    return $self;
}

sub _clear
{
    my $self = shift(@_);

    # GUI elements.
    $self->{window} = undef;
    $self->{slist} = undef;
    $self->{statusbar} = undef;

    # Data elements.
    $self->{name} = undef;
    $self->{slist_sort_column} = 0;
    $self->{slist_sort_order} = $SORT_ASCENDING;
    $self->{summaries} = [];
    $self->{visible_summaries} = [];
    $self->{hide_active} = 0;
    $self->{hide_inactive} = 0;

    return 1;
}

sub run
{
    my $self = shift(@_);

    $self->{window}->show_all;
    Gtk2->main;

    return 1;
}

# Convenience function.
sub _build_menu ()
{
    my $self = shift(@_);

    my $menu_tree
	= [
	   '_File' => {
	       item_type  => '<Branch>',
	       children => [
			    '_Quit' => {
				callback => sub { Gtk2->main_quit; },
				callback_action => 10,
				accelerator => '<ctrl>Q'
				}
			    ]
			    },
	   '_Settings' => {
	       item_type  => '<Branch>',
	       children => [
			    'Hide _Active Addresses' => {
				item_type => '<CheckItem>',
				callback => \&_hide_active,
				callback_action => 20
			    },
			    'Hide _Inactive Addresses' => {
				item_type => '<CheckItem>',
				callback => \&_hide_inactive,
				callback_action => 21
			    },
			    '_Preferences...' => {
				callback_action => 22
			    }
			    ]
			},
	   '_Help' => {
	       item_type  => '<Branch>',
	       children => [
			    '_About...' => {
				callback => \&_show_about,
				callback_action => 30
				}
			    ]
			    }
	   ];

    return Gtk2::SimpleMenu->new(menu_tree => $menu_tree,
				 user_data => $self);
}

# Gtk+ callback.
sub _column_clicked ($$)
{
    my ($self, $guiref) = @_;
    my $slist = $guiref->{slist};
    my $summaries_ref = $guiref->{visible_summaries};

    bless $self, "Gtk2::TreeViewColumn";

    my @columns = $slist->get_columns();
    my $n = 0;
    my $found = 0;

    for ($n = 0; $n <= $#columns; $n++)
    {
	if (!$found && ($columns[$n]->get_title() eq $self->get_title()))
	{
	    $found = 1;
	    last;
	}
    }

    if (!$found)
    {
	return;
    }

    my $column;
    foreach $column (@columns)
    {
	$column->set_sort_indicator(0);
    }

    $self->{slist_sort_column} = $n;
    if ($self->get_sort_order() eq $SORT_ASCENDING)
    {
	$self->{slist_sort_order} = $SORT_DESCENDING;
    }
    else
    {
	$self->{slist_sort_order} = $SORT_ASCENDING;
    }

    $guiref->_sort_summaries();

    return 1;
}

# Gtk+ callback.
sub _goto_clicked ($$)
{
    my ($self, $ref) = @_;
    my $guiref = ${$ref}[0];
    my $slist = $guiref->{slist};
    my $where = ${$ref}[1];
    my $ret = 0; # Fail by default.

    # Path manipulation doesn't seem to work correctly.
    # $path->next() doesn't return a bool, for instance.
    # Go via integers instead. Ugh.
    my ($path) = $slist->get_cursor();
    if (defined($path))
    {
	my ($idx) = $path->get_indices();

	if ($where eq 'top')
	{
	    $idx = 0;
	}
	elsif ($where eq 'up')
	{
	    if ($idx > 0)
	    {
		$idx--;
	    }
	}
	elsif ($where eq 'down')
	{
	    if ($idx < $#{$slist->{data}})
	    {
		$idx++;
	    }
	}
	elsif ($where eq 'bottom')
	{
	    $idx = $#{$slist->{data}};
	}

	$path = Gtk2::TreePath->new_from_indices($idx);
	if (defined($path))
	{
	    $slist->set_cursor($path);
	    $ret = 1;
	}
    }

    return $ret;
}

sub _show_properties ($$)
{
    my ($self, $ref) = @_;
    my $guiref = $ref;
    my $window = $guiref->{window};
    my $slist = $guiref->{slist};
    my $lists_ref = $guiref->{subscriptions}->lists();
    my $sel = $slist->get_selection();
    my @paths = $sel->get_selected_rows();
    my $path;

    # Currently we can only select one list at a time.
    # When we can select more than one list, popping up a dialog box
    # per list will not work very well. Perhaps have some kind of selector
    # that show the details for that particular list.
    foreach $path (@paths)
    {
	my ($idx) = $path->get_indices();
	my $listname = $slist->{data}->[$idx]->[0];
	my $subscriber = $slist->{data}->[$idx]->[1];
	# TODO: FIXME: Ugly. This shouldn't dive directly in.
	# Wrap the subscription in a class.
	my $subscriptions = $lists_ref->{$listname}->{subscriptions};
	my $subscription;
	my $i;

	# Find the original subscription information.
	for ($i = 0; $i <= $#{$subscriptions}; $i++)
	{
	    # ASSUMPTION: Each address is only subscribed once
	    # to a list.
	    if ($subscriptions->[$i]->{subscriber} eq $subscriber)
	    {
		$subscription = $subscriptions->[$i];
		last;
	    }
	}

	defined($subscription) || next;

	# Pop up a properties dialog here.
	my $dialog
	    = Gtk2::Dialog->new_with_buttons('Properties for '.$subscription->{listname},
					     $window,
					     'destroy-with-parent',
					     'gtk-ok' => 'none');

	my $table = Gtk2::Table->new(3, 2);
	$dialog->vbox->add($table);

	my @labels;
	push(@labels, Gtk2::Label->new('List:'));
	push(@labels, Gtk2::Label->new('Subscriber address:'));
	push(@labels, Gtk2::Label->new('Subscribed:'));
	push(@labels, Gtk2::Label->new('Since:'));
	push(@labels, Gtk2::Label->new('URL:'));
	push(@labels, Gtk2::Label->new('Password:'));

	my @data;
	push(@data, Gtk2::Label->new($subscription->{listname}));
	push(@data, Gtk2::Label->new($subscription->{subscriber}));
	push(@data, Gtk2::Label->new($subscription->{type} == $Email::List::Subscription::SUBSCRIPTION ? 'Yes' : 'No'));

	push(@data, Gtk2::Label->new(defined($subscription->{date})
				     ? $subscription->{date}
				     : '?'));

	push(@data, Gtk2::Label->new(defined($subscription->{url})
				     ? $subscription->{url}
				     : '(None)'));

	push(@data, Gtk2::Label->new(defined($subscription->{password})
				     ? '(Available)'
				     : '(Unknown or none)'));

	for ($i = 0; $i <= $#labels; $i++)
	{
	    $table->attach_defaults($labels[$i], 0, 1, $i, $i + 1);
	    $table->attach_defaults($data[$i], 1, 2, $i, $i + 1);
	}

	$dialog->show_all();
	$dialog->signal_connect(response => sub { $_[0]->destroy });
	$dialog->run();
    }
}

# Convenience function.
sub _prune_summaries
{
   my $self = shift(@_);
   my $ref = $self->{summaries};
   my @l;
   my $summary;

   foreach $summary (@{$ref})
   {
       my $hide = 0;

       # TODO: Yuck, use objects to encapsulate rather than indexing.
       if (   $self->{hide_inactive}
	   && $self->{config}->is_inactive($summary->[1]))
       {
	   $hide = 1;
       }

       if (   $self->{hide_active}
	   && !$self->{config}->is_inactive($summary->[1]))
       {
	   $hide = 1;
       }

       # Hide this address, if we should.
       push(@l, $summary) unless $hide;
   }

   @{$self->{visible_summaries}} = @l;
   @{$self->{slist}->{data}} = @l;

   return 1;
}

# Convenience function.
sub _sort_summaries
{
    my $self = shift(@_);
    my $ref = $self->{visible_summaries};
    my $n = $self->{slist_sort_column};
    my $order = $self->{slist_sort_order};
    my @l;

    if ($order eq $SORT_ASCENDING)
    {
	@l = sort { $a->[$n] cmp $b->[$n] } @{$ref};
    }
    else
    {
	@l = sort { $b->[$n] cmp $a->[$n] } @{$ref};
    }

    # Update lists.
    @{$self->{visible_summaries}} = @l;
    @{$self->{slist}->{data}} = @l;

    return 1;
}

# Convenience function.
sub _set_status ($)
{
    my $self = shift(@_);
    my $n = $#{$self->{summaries}};
    my $n_visible = $#{$self->{visible_summaries}};

    $n = $n > 0 ? $n : 0;
    $n_visible = $n_visible > 0 ? $n_visible : 0;

    my $n_hidden = $n - $n_visible;
    my $n_visible_plural = ($n_visible == 0) || ($n_visible > 1) ? 's' : '';
    my $n_hidden_plural = ($n_hidden == 0) || ($n_hidden > 1) ? 's' : '';

    $self->{statusbar}->pop(0);
    $self->{statusbar}->push(0,
			     $n_visible.' list'.$n_visible_plural.' shown; '
			     .$n_hidden.' list'.$n_hidden_plural.' hidden');
    return 1;
}

# Convenience function.
sub _update_summaries ($)
{
    my $self = shift(@_);

    $self->_prune_summaries();
    $self->_sort_summaries();
    $self->_set_status();

    return 1;
}

sub set_summaries
{
    my $self = shift(@_);
    my $summaries_ref = shift(@_);

    $self->{summaries} = [ @{$summaries_ref} ];

    return $self->_update_summaries();
}

sub _show_about ($)
{
    my ($ref) = @_;
    my $guiref = $ref;
    my $window = $guiref->{window};

    # Pop up a properties dialog here.
    my $dialog
	= Gtk2::Dialog->new_with_buttons('About '.$guiref->{name},
					 $window,
					 'destroy-with-parent',
					 'gtk-ok' => 'none');

    my $label = Gtk2::Label->new($guiref->{name}
				 .', version '.$Subscripto::Version::VERSION);
    $dialog->vbox->add($label);
    $label = Gtk2::Label->new($Subscripto::Version::COPYRIGHT
			      .' by '
			      .$Subscripto::Version::AUTHOR);
    $dialog->vbox->add($label);

    $dialog->show_all();
    $dialog->signal_connect(response => sub { $_[0]->destroy });
    $dialog->run();
}

sub _hide_active ($)
{
    my ($ref) = @_;
    my $guiref = $ref;
    my $menu = $guiref->{menu};
    my $menu_item = $menu->get_widget('/Settings/Hide Active Addresses');

    # Sanity check.
    defined($menu_item) || die 'Unable to find Hide Active Addresses';

    if ($menu_item->get_active())
    {
	# Hide subscriptions for active addresses.
	$guiref->{hide_active} = 1;
	$guiref->_update_summaries();
    }
    else
    {
	# Show all subscriptions.
	$guiref->{hide_active} = 0;
	$guiref->_update_summaries();
    }
}

sub _hide_inactive ($)
{
    my ($ref) = @_;
    my $guiref = $ref;
    my $menu = $guiref->{menu};
    my $menu_item = $menu->get_widget('/Settings/Hide Inactive Addresses');

    # Sanity check.
    defined($menu_item) || die 'Unable to find Hide Inactive Addresses';

    if ($menu_item->get_active())
    {
	# Hide subscriptions for inactive addresses.
	$guiref->{hide_inactive} = 1;
	$guiref->_update_summaries();
    }
    else
    {
	# Show all subscriptions.
	$guiref->{hide_inactive} = 0;
	$guiref->_update_summaries();
    }
}

1;
